@@ -1,6 +1,6 @@
 ;;; sb-asahi.el --- shimbun backend for asahi.com -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2011 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001-2011, 2013, 2014 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -146,7 +146,7 @@ Every `.' in NAME will be replaced with `/'."
 		"\\([^/]+\\)"
 		"\\(?:/update/"
 		;; 3 and 4. serial number
-		"\\([0-9]+\\)\\)?/\\([a-z]*[0-9]+\\)"
+		"\\([0-9]+\\)\\)?/\\([0-9a-z]+\\)"
 		"\\.html\\?ref=rss\\)"
 		"\"" s0 ">" s0 "<title>" s0
 		;; 5. subject
@@ -219,20 +219,16 @@ Every `.' in NAME will be replaced with `/'."
        1 nil 2 6 3 4 5)
       ("digital" "$B%G%8%?%k(B" "%s/list.html"
        ,@(shimbun-asahi-make-regexp "digital/[^\"/]+"))
-      ("editorial" "$B<R@b(B" "include/editorial_bno4.xml"
-       ,(concat
-	 "<a" s1 "href=\"\\./"
-	 ;; 1. url
-	 "\\(editorial"
-	 ;; 2. year
-	 "\\(20[0-9][0-9]\\)"
-	 ;; 3. month
-	 "\\([01][0-9]\\)"
-	 ;; 4. day
-	 "\\([0-3][0-9]\\)"
-	 "\\.html\\)"
-	 "\"")
-       1 nil nil nil 2 3 4)
+      ("editorial" "$B<R@b(B" "paper/editorial.html"
+       ,(concat "<meta" s1 "\\(?:[^\t\n >]+" s1 "\\)*name=\"RELEASE_DATE\""
+		s1 "\\(?:[^\t\n >]+" s1 "\\)*CONTENT=\""
+		;; 1. year
+		"\\(20[1-9][0-9]\\)-"
+		;; 2. month
+		"\\([01]?[0-9]\\)-"
+		;; 3. day
+		"\\([0-3]?[0-9]\\)\"")
+       nil nil nil nil 1 2 3)
       ("edu" "$B650i(B" "%s/list.html" ,@edu)
       ("english" "ENGLISH" "%s/index.html"
        ,@(let ((rest (shimbun-asahi-make-regexp "english.Herald-asahi")))
@@ -279,20 +275,16 @@ Every `.' in NAME will be replaced with `/'."
       ("sports.rugby" "$B%i%0%S!<(B")
       ("sports.usa" "$BJF%W%m%9%]!<%D(B")
       ("sports.winter" "$B%&%$%s%?!<%9%]!<%D(B")
-      ("tenjin" "$BE7@<?M8l(B" "include/column_bno4.xml"
-       ,(concat
-	 "<a" s1 "href=\"\\./"
-	 ;; 1. url
-	 "\\(column"
-	 ;; 2. year
-	 "\\(20[0-9][0-9]\\)"
-	 ;; 3. month
-	 "\\([01][0-9]\\)"
-	 ;; 4. day
-	 "\\([0-3][0-9]\\)"
-	 "\\.html\\)"
-	 "\"")
-       1 nil nil nil 2 3 4)
+      ("tenjin" "$BE7@<?M8l(B" "paper/column.html"
+       ,(concat "<meta" s1 "\\(?:[^\t\n >]+" s1 "\\)*name=\"RELEASE_DATE\""
+		s1 "\\(?:[^\t\n >]+" s1 "\\)*CONTENT=\""
+		;; 1. year
+		"\\(20[1-9][0-9]\\)-"
+		;; 2. month
+		"\\([01]?[0-9]\\)-"
+		;; 3. day
+		"\\([0-3]?[0-9]\\)\"")
+       nil nil nil nil 1 2 3)
       ("travel" "$B%H%i%Y%k(B" "%s/news/"
        ,@(shimbun-asahi-make-regexp "travel.news"))
       ("wakata" "$B<cED$5$s$-$\$&BZ:_5-(B" "%s/"
@@ -933,39 +925,16 @@ Each table is the same as the `cdr' of the element of
 `shimbun-asahi-group-table'.")
 
 (defvar shimbun-asahi-content-start
-  "<div[\t\n ]+class=\"\
-\\(?:ThmbSet300Tb\\|ThmbSet256\\|Kansai-ThmbSet100\\|ThmbCol\\)\">\
-\\|<!--[\t\n ]*End of Headline[\t\n ]*-->\
-\\(?:[\t\n ]*<div[\t\n ]+[^<]+</div>[\t\n ]*\
-\\|[\t\n ]*<p[\t\n ]+[^<]+</p>[\t\n ]*\\)?\
-\\|<!--[\t\n ]*Start of \\(Kiji\\|photo\\)[\t\n ]*-->\
-\\|<!--[\t\n ]*FJZONE START NAME=\"HONBUN\"[\t\n ]*-->")
+  "<!-+[\t\n ]*ArticleBody[\t\n ]*BGN[\t\n ]*-+>")
 
 (defvar shimbun-asahi-content-end
-  "<dl[\t\n ]+class=\"PrInfo\">\
-\\|<!--[\t\n ]*google_ad_section_end\
-\\|<!-[^>]+[^>$B!z(B]$B$3$3$^$G(B[\t\n ]*-+>\
-\\|\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
-\\|<!--[\t\n ]*End of Kiji[\t\n ]*-->\
-\\|<!--[\t\n ]*End of related link[\t\n ]*-->\
-\\|<!--[\t\n ]*FJZONE END NAME=\"HONBUN\"[\t\n ]*-->")
+  "<!-+[\t\n ]*ArticleBody[\t\n ]*END[\t\n ]*-+>")
 
 (defvar shimbun-asahi-text-content-start
-  "<div[\t\n ]+class=\"\\(?:ThmbSet256\\|Kansai-ThmbSet100\\|ThmbCol\\)\">\
-\\|<!--[\t\n ]*End of Headline[\t\n ]*-->\
-\\(?:[\t\n ]*<div[\t\n ]+[^<]+</div>[\t\n ]*\
-\\|[\t\n ]*<p[\t\n ]+[^<]+</p>[\t\n ]*\\)?\
-\\|<!--[\t\n ]*Start of Kiji[\t\n ]*-->\
-\\|<!--[\t\n ]*FJZONE START NAME=\"HONBUN\"[\t\n ]*-->")
+  "<!-+[\t\n ]*ArticleText[\t\n ]*BGN[\t\n ]*-+>")
 
 (defvar shimbun-asahi-text-content-end
-  "<dl[\t\n ]+class=\"PrInfo\">\
-\\|<!--[\t\n ]*google_ad_section_end\
-\\|<!-[^>]+$B$3$3$^$G(B[\t\n ]*-+>\
-\\|\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
-\\|<!--[\t\n ]*\\(?:google_ad_section\\|[AD$B!z!y(B]+\\)\
-\\|<!--[\t\n ]*End of Kiji[\t\n ]*-->\
-\\|<!--[\t\n ]*FJZONE END NAME=\"HONBUN\"[\t\n ]*-->")
+  "<!-+[\t\n ]*ArticleText[\t\n ]*END[\t\n ]*-+>")
 
 (defvar shimbun-asahi-x-face-alist
   '(("default" . "\
@@ -1013,6 +982,71 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 	  (t
 	   (shimbun-expand-url (format index group) shimbun-asahi-url)))))
 
+(defun shimbun-asahi-article-contents (group)
+  "Get article's contents for editorial and tenjin groups.
+Contents will be saved in the shimbun header as the extra element."
+  (let ((case-fold-search t)
+	contents)
+    (cond ((string-equal group "editorial")
+	   (goto-char (point-max))
+	   (shimbun-retrieve-url "http://www.asahi.com/paper/editorial2.html")
+	   (let (tem)
+	     (goto-char (point-min))
+	     (while (and (re-search-forward "\
+<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleTitle\"" nil t)
+			 (re-search-forward "\
+<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"Title\"" nil t)
+			 (shimbun-end-of-tag "div")
+			 (progn
+			   (goto-char (match-beginning 2))
+			   (if (re-search-forward "<h[0-9]>[\t\n ]*\\([^<]+\\)"
+						  (match-end 2) t)
+			       (progn
+				 (push (shimbun-replace-in-string
+					(match-string 1) "[\t ]+\\'" "")
+				       tem)
+				 (re-search-forward "\
+<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleText\"" nil t))
+			     (goto-char (match-end 0))
+			     nil))
+			 (shimbun-end-of-tag "div" t))
+	       (push (match-string 3) tem))
+	     (goto-char (point-max))
+	     (setq tem (nreverse tem))
+	     (while tem
+	       (setq contents (concat contents
+				      "<p><h2>" (pop tem) "</h2></p>\n"
+				      (pop tem) "\n")))))
+	  ((string-equal group "tenjin")
+	   (goto-char (point-min))
+	   (when (and (re-search-forward "\
+<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleBody\"" nil t)
+		      (search-forward "$B"'(B" nil t)
+		      (re-search-backward
+		       "<p\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]*>" nil t)
+		      (shimbun-end-of-tag "p" t))
+	     (setq contents
+		   (concat "<p>\n"
+			   (mapconcat 'identity
+				      (split-string (match-string 3)
+						    "[\t\n $B!!(B]*$B"'(B[\t\n $B!!(B]*" t)
+				      "$B!#(B\n</p>\n<p>\n$B!!(B")
+			   "\n</p>\n")))
+	   (goto-char (point-max))))
+    (with-temp-buffer
+      (set-buffer-multibyte nil)
+      (insert (if contents
+		  (encode-coding-string contents 'utf-8)
+		"<h2>No content retrieved.</h2>\n"))
+      (let ((coding-system-for-read 'binary)
+	    (coding-system-for-write 'binary))
+	(shell-command-on-region (point-min) (point-max) "gzip -f9" nil t))
+      (base64-encode-region (point-min) (point-max))
+      (goto-char (point-min))
+      (while (re-search-forward "[\n\r]+" nil t)
+	(delete-region (match-beginning 0) (match-end 0)))
+      `((Contents . ,(buffer-string))))))
+
 (defun shimbun-asahi-get-headers (shimbun)
   "Return a list of headers."
   (let ((group (shimbun-current-group-internal shimbun))
@@ -1054,6 +1088,7 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 <h[0-9]\\(?:[\n\t ]+[^>]+\\)?>[\t\n ]*\\([^&]+\\)[\t\n ]*&#[0-9]+"
 						     nil t)
 				 (downcase (match-string 1)))))))
+		  (paper-p)
 		  (t
 		   (setq hour-min
 			 (save-excursion
@@ -1108,6 +1143,7 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 				 shimbun-asahi-top-level-domain ">")
 		       (concat "<" serial "%" rgroup "."
 			       shimbun-asahi-top-level-domain ">")))
+	    (setq extra (and paper-p (shimbun-asahi-article-contents group)))
 	    (unless (shimbun-search-id shimbun id)
 	      (push (shimbun-create-header
 		     ;; number
@@ -1168,18 +1204,21 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 		     ;; references, chars, lines
 		     "" 0 0
 		     ;; xref
-		     (shimbun-expand-url
-		      (match-string (nth 0 numbers))
-		      (cond (paper-p
-			     (concat shimbun-asahi-url "paper/"))
-			    (book-p
-			     "http://book.asahi.com/")
-			    ((string-match "$BA%66MN0l$N@$3&%V%j!<%U%#%s%0(B" from)
-			     "http://opendoors.asahi.com/syukan/briefing/")
-			    (iraq-p
-			     "http://www2.asahi.com/")
-			    (t
-			     shimbun-asahi-url))))
+		     (if paper-p
+			 (shimbun-index-url shimbun)
+		       (shimbun-expand-url
+			(match-string (nth 0 numbers))
+			(cond (book-p
+			       "http://book.asahi.com/")
+			      ((string-match "$BA%66MN0l$N@$3&%V%j!<%U%#%s%0(B"
+					     from)
+			       "http://opendoors.asahi.com/syukan/briefing/")
+			      (iraq-p
+			       "http://www2.asahi.com/")
+			      (t
+			       shimbun-asahi-url))))
+		     ;; extra
+		     extra)
 		    headers)))
 	  (when (string-match "$B;d$N%_%+%?(B" from)
 	    (setq headers (nreverse headers))))
@@ -1194,59 +1233,12 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 		    numbers (cdddar subgroups)
 		    subgroups (cdr subgroups)))
 	  (throw 'stop nil))))
-    (append (shimbun-sort-headers headers)
-	    (shimbun-asahi-get-headers-for-today group jname from))))
+    (shimbun-sort-headers headers)))
 
 (luna-define-method shimbun-get-headers ((shimbun shimbun-asahi)
 					 &optional range)
   (shimbun-asahi-get-headers shimbun))
 
-(defun shimbun-asahi-get-headers-for-today (group jname from)
-  "Return a list of the header for today's article.
-It works for only the groups `editorial' and `tenjin'."
-  (goto-char (point-min))
-  (let ((basename (cdr (assoc group '(("editorial" . "editorial")
-				      ("tenjin" . "column")))))
-	year cmonth month day url)
-    (when (and basename
-	       (re-search-forward
-		(eval-when-compile
-		  (concat
-		   "/\\(?:editorial\\|column\\)\\.html\"[^\n0-9]+"
-		   ;; 1. month
-		   "\\([01]?[0-9]\\)" "$B7n(B"
-		   ;; 2. day
-		   "\\([0-3]?[0-9]\\)" "$BF|IU(B"))
-		nil t))
-      (setq year (shimbun-decode-time nil 32400)
-	    cmonth (nth 4 year)
-	    year (nth 5 year)
-	    month (string-to-number (match-string 1))
-	    day (string-to-number (match-string 2)))
-      (cond ((and (= cmonth 1) (= month 12))
-	     (decf year))
-	    ((and (= cmonth 12) (= month 1))
-	     (incf year)))
-      (setq url (format "paper/%s%d%02d%02d.html" basename year month day))
-      (list
-       (shimbun-make-header
-	;; number
-	0
-	;; subject
-	(shimbun-mime-encode-string (concat jname
-					    (format " (%d/%d)" month day)))
-	;; from
-	from
-	;; date
-	(shimbun-make-date-string year month day "07:00")
-	;; id
-	(format "<%d%02d%02d%%%s.%s>"
-		year month day group shimbun-asahi-top-level-domain)
-	;; references, chars, lines
-	"" 0 0
-	;; xref
-	(shimbun-expand-url url shimbun-asahi-url))))))
-
 (defun shimbun-asahi-multi-next-url (shimbun header url)
   (goto-char (point-min))
   (when (and (re-search-forward "\
@@ -1270,10 +1262,47 @@ It works for only the groups `editorial' and `tenjin'."
 					    header url)
   (shimbun-asahi-multi-next-url shimbun header url))
 
+(eval-when-compile
+  (unless (fboundp 'zlib-decompress-region)
+    (defalias 'zlib-decompress-region 'ignore)))
+
+(defun shimbun-asahi-article (shimbun header outbuf)
+  "Fetch article contents."
+  (if (member (shimbun-current-group-internal shimbun) '("editorial" "tenjin"))
+      (let ((contents (cdr (assq 'Contents (shimbun-header-extra header)))))
+	(insert
+	 (with-temp-buffer
+	   (if contents
+	       (progn
+		 (set-buffer-multibyte nil)
+		 (insert contents)
+		 (base64-decode-region (point-min) (point-max))
+		 (if (and (fboundp 'zlib-decompress-region)
+			  (not (eq (symbol-function 'zlib-decompress-region)
+				   'ignore)))
+		     (zlib-decompress-region (point-min) (point-max))
+		   (let ((coding-system-for-read 'binary)
+			 (coding-system-for-write 'binary))
+		     (shell-command-on-region (point-min) (point-max)
+					      "gzip -d" nil t)))
+		 (setq contents (decode-coding-string (buffer-string) 'utf-8))
+		 (erase-buffer)
+		 (set-buffer-multibyte t)
+		 (insert "<div class=\"ArticleBody\">"
+			 contents
+			 "</div><!-- ArticleBody END -->"))
+	     (insert "<h2>No content found.</h2>\n"))
+	   (shimbun-make-contents shimbun header))))
+    (luna-call-next-method)))
+
+(luna-define-method shimbun-article :around ((shimbun shimbun-asahi)
+					     header &optional outbuf)
+  (shimbun-asahi-article shimbun header outbuf))
+
 (defun shimbun-asahi-prepare-article (shimbun header)
   "Prepare an article.
-For the groups editorial and tenjin, it tries to fetch the article for
-that day if it failed."
+For the groups editorial and tenjin, use the extra element of HEADER
+as article contents."
   (let ((case-fold-search t)
 	(group (shimbun-current-group-internal shimbun))
 	(from (shimbun-header-from-internal header)))
@@ -1305,49 +1334,8 @@ that day if it failed."
 					nil t)
 		 (goto-char (match-beginning 0))
 		 (insert "\n<!-- End of Kiji -->\n"))))))
-     ((string-equal group "editorial")
-      (let ((url (shimbun-header-xref header))
-	    (retry 0)
-	    start)
-	(while retry
-	  (if (progn
-		(while (and (re-search-forward "\
-<\\(h[0-9]+\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"bdr_btm_2px"
-					       nil t)
-			    (shimbun-end-of-tag (match-string 1) t)
-			    (if start
-				(progn
-				  (delete-region
-				   (goto-char (match-beginning 0))
-				   (match-beginning 1))
-				  (insert "\n&#012;\n")
-				  (forward-char 1))
-			      (setq start (match-beginning 1)))))
-		(and start
-		     (re-search-forward "[\t\n ]*\
-\\(?:<ul[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"FollowLnk\
-\\|<li[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"TopLnk\"\
-\\|<dl[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"PrInfo\"\\)"
-					nil t)))
-	      (progn
-		(goto-char (match-beginning 0))
-		(setq retry nil)
-		(insert "\n<!-- End of Kiji -->")
-		(goto-char start)
-		(insert "<!-- Start of Kiji -->")
-		(when (string-match "/editorial\\.html\\'" url)
-		  (insert "\
-\n<p>($B;XDj$5$l$?(B&nbsp;url&nbsp;$B$,(B&nbsp;$B$^$@(B/$B$9$G$K(B&nbsp;$BL5$$$N$G!"(B\
-<a href=\"" url "\">$B%H%C%W%Z!<%8(B</a> $B$+$i5-;v$r<hF@$7$^$7$?(B)</p>\n")))
-	    (erase-buffer)
-	    (if (= retry 1)
-		(setq retry nil)
-	      (setq url "http://www.asahi.com/paper/editorial.html"
-		    retry 1
-		    start nil)
-	      (shimbun-header-set-xref header url)
-	      (shimbun-fetch-url shimbun url)
-	      (goto-char (point-min)))))))
+     ((member group '("editorial" "tenjin"))
+      )
      ((string-equal group "food")
       (when (and (re-search-forward (shimbun-content-start shimbun) nil t)
 		 (re-search-forward "[\t\n ]*<!-+[\t\n ]+Creative[\t\n ]+for"
@@ -1369,47 +1357,6 @@ that day if it failed."
 	  (goto-char (match-beginning 0))
 	  (insert "<!-- End of Kiji -->"))))
      ((string-equal group "rss"))
-     ((string-equal group "tenjin")
-      (let ((url (shimbun-header-xref header))
-	    (retry 0)
-	    start)
-	(while retry
-	  (if (and (re-search-forward "\
-<p>20[0-9][0-9]$BG/(B[01]?[0-9]$B7n(B[0-3]?[0-9]$BF|!J(B[$BF|7n2P?eLZ6bEZ(B]$B!KIU(B</p>\
-\\(\\(?:[\t\n ]*$B0u:~(B\\)?[\t\n ]*<[^>]+>\\)+[\t\n ]*"
-				      nil t)
-		   (progn
-		     (setq start (match-end 0))
-		     (re-search-forward "[\t\n ]*</p>" nil t)))
-	      (save-restriction
-		(narrow-to-region start (goto-char (match-beginning 0)))
-		(setq retry nil)
-		(insert "</p>\n<!-- End of Kiji -->")
-		(goto-char start)
-		(insert "<!-- Start of Kiji --><p>")
-		(when (string-match "/column\\.html\\'" url)
-		  (insert "\
-\n<p>($B;XDj$5$l$?(B&nbsp;url&nbsp;$B$,(B&nbsp;$B$^$@(B/$B$9$G$K(B&nbsp;$BL5$$$N$G!"(B\
-<a href=\"" url "\">$B%H%C%W%Z!<%8(B</a> $B$+$i5-;v$r<hF@$7$^$7$?(B)</p>\n"))
-		(while (re-search-forward
-			(eval-when-compile
-			  (concat "[$B"'(B"
-				  (condition-case nil
-				      (list (make-char 'mule-unicode-2500-33ff
-						       33 124))
-				    (error nil))
-				  "]"))
-			nil t)
-		  (replace-match "$B!#(B</p>\n<p>$B!!(B")))
-	    (erase-buffer)
-	    (if (= retry 1)
-		(setq retry nil)
-	      (setq url "http://www.asahi.com/paper/column.html"
-		    retry 1
-		    start nil)
-	      (shimbun-header-set-xref header url)
-	      (shimbun-fetch-url shimbun url)
-	      (goto-char (point-min)))))))
      ((string-match "\\`book\\(?:\\.\\|\\'\\)" group)
       (while (re-search-forward "\\(<a[\t\n ]+[^>]+>\\)\
 \[\t\n ]*<img[\t\n ]+[^>]+>[\t\n ]*</a>"
@@ -1569,14 +1516,27 @@ that day if it failed."
 \\|<small>[^<]+</small>"))
 		(not (eq (char-after) ?<)))
 	(replace-match "\\1<br>\n")))
-    ;; Remove related topics.
+    ;; Remove related topics, etc.
     (goto-char (point-min))
     (while (re-search-forward "\
-<\\(div\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"LnkRelated\\(?:AsaD\\)?\"\
+<\\(li\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ReadMore\"\
+\\|<\\(div\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\
+\"\\(?:RelatedLi\\(?:nk\\|st\\)Mod\\|PrTextMod\\|LnkRelated\\(?:AsaD\\)\\)?\"\
 \\|<\\(p\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"AsaDLnk\"" nil t)
-      (when (shimbun-end-of-tag (or (match-string 1) (match-string 2)) t)
+      (when (shimbun-end-of-tag
+	     (or (match-string 1) (match-string 2) (match-string 3)) t)
 	(delete-region (match-beginning 0) (match-end 0))
 	(insert "\n")))
+    ;; Remove Ads.
+    (goto-char (point-min))
+    (let (st)
+      (while (and (re-search-forward
+		   "[\t\n ]*<!-+[\t\n ]*Ad[\t\n ]+BGN[\t\n ]*-+>" nil t)
+		  (progn
+		    (setq st (match-beginning 0))
+		    (re-search-forward
+		     "<!-+[\t\n ]*Ad[\t\n ]+END[\t\n ]*-+>[\t\n ]*" nil t)))
+	(delete-region st (match-end 0))))
     (unless (shimbun-prefer-text-plain-internal shimbun)
       (shimbun-break-long-japanese-lines))
     t))
