@@ -1,6 +1,6 @@
 ;;; w3mhack.el --- a hack to setup the environment for building w3m
 
-;; Copyright (C) 2001-2010, 2012
+;; Copyright (C) 2001-2010, 2012, 2013
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: Katsumi Yamaoka <yamaoka@jpl.org>
@@ -289,7 +289,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 	 (version-specific-modules '("w3m-ems.el" "w3m-xmas.el"))
 	 (ignores;; modules not to be byte-compiled.
 	  (append
-	   (list "w3mhack.el" "w3m-setup.el" w3mhack-load-file)
+	   (list "w3mhack.el")
 	   (delete (if (featurep 'xemacs) "w3m-xmas.el" "w3m-ems.el")
 		   (copy-sequence version-specific-modules))))
 	 (shimbun-dir (file-name-as-directory shimbun-module-directory))
@@ -395,31 +395,31 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
   (and (not w3mhack-nonunix-icondir)
        (setq w3mhack-nonunix-icondir
 	     (expand-file-name "images/w3m" data-directory)))
-  (labels
-      ((mkdir (dir)
-	      (unless (file-directory-p dir)
-		(message "mkdir %s" dir)
-		(unless w3mhack-nonunix-dryrun
-		  (make-directory dir 'parents))))
-       (install (srcdir dstdir pattern)
-		(dolist (src (directory-files srcdir t pattern))
-		  (let ((dst (expand-file-name
-			      (file-name-nondirectory src) dstdir)))
-		    (message "cp %s %s"
-			     (file-relative-name src default-directory) dst)
-		    (unless w3mhack-nonunix-dryrun
-		      (copy-file src dst t t))))))
-    (mkdir w3mhack-nonunix-lispdir)
-    (install default-directory w3mhack-nonunix-lispdir "\\.elc?\\'")
+  (let ((mkdir (lambda (dir)
+		 (unless (file-directory-p dir)
+		   (message "mkdir %s" dir)
+		   (unless w3mhack-nonunix-dryrun
+		     (make-directory dir 'parents)))))
+	(install (lambda (srcdir dstdir pattern)
+		   (dolist (src (directory-files srcdir t pattern))
+		     (let ((dst (expand-file-name
+				 (file-name-nondirectory src) dstdir)))
+		       (message "cp %s %s"
+				(file-relative-name src default-directory) dst)
+		       (unless w3mhack-nonunix-dryrun
+			 (copy-file src dst t t)))))))
+    (funcall mkdir w3mhack-nonunix-lispdir)
+    (funcall install default-directory w3mhack-nonunix-lispdir "\\.elc?\\'")
     (let ((shimbun-directory
 	   (expand-file-name shimbun-module-directory default-directory)))
       (when (file-exists-p (expand-file-name "shimbun.elc" shimbun-directory))
-	(install shimbun-directory w3mhack-nonunix-lispdir "\\.elc?\\'")))
+	(funcall install shimbun-directory w3mhack-nonunix-lispdir
+		 "\\.elc?\\'")))
     (when w3mhack-nonunix-icondir
-      (mkdir w3mhack-nonunix-icondir)
-      (install (expand-file-name (if (featurep 'xemacs)
-				     "icons30"
-				   "icons"))
+      (funcall mkdir w3mhack-nonunix-icondir)
+      (funcall install (expand-file-name (if (featurep 'xemacs)
+					     "icons30"
+					   "icons"))
 	       w3mhack-nonunix-icondir "\\.\\(?:png\\|xpm\\)\\'"))))
 
 ;; Byte optimizers and version specific functions.
@@ -612,8 +612,12 @@ to remove some obsolete variables in the first argument VARLIST."
 	;; This system doesn't allow hard links.
 	(setq make-hardlink 'copy-file))
       (dolist (el (cons w3mhack-load-file (w3mhack-module-list)))
-	(funcall make-hardlink
-		 el (expand-file-name (file-name-nondirectory el) temp-dir)))
+	(setq hardlink (expand-file-name (file-name-nondirectory el) temp-dir))
+	(or (file-exists-p hardlink)
+	    ;; w3m-load.el may have been already generated.
+	    (funcall make-hardlink
+		     el (expand-file-name (file-name-nondirectory el)
+					  temp-dir))))
       (with-temp-buffer
 	(let ((standard-output (current-buffer)))
 	  (Custom-make-dependencies temp-dir))
@@ -979,6 +983,14 @@ NOTE: This function must be called from the top directory."
 	  (goto-char (point-max))
 	  (insert (if (featurep 'xemacs) "\C-l\n" "")
 		  ";;; " w3mhack-load-file " ends here\n"))
+
+	;; Delete "no-byte-compile" so that w3m-load.el will byte compile.
+	;; Byte compiling gives dynamic docstrings which means they aren't
+	;; loaded into memory unless or until required.
+	(goto-char (point-min))
+	(when (re-search-forward "^;; no-byte-compile: t\n" nil t)
+	  (delete-region (match-beginning 0) (match-end 0)))
+
 	(save-buffer)))))
 
 (defun w3mhack-generate-xemacs-load-file (file)
